/*
 * NAT - An universal Translator
 * Copyright (C) 2005 Bruno Mascret
 * Contact: bmascret@free.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package nat;

import gestionnaires.GestionnaireErreur;

import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.nio.channels.FileChannel;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.Properties;
import nat.OptNames;
import java.lang.Boolean;
import java.lang.Integer;
import java.io.File;
import java.util.ArrayList;
//import javax.print.PrintService;

//import gestionnaires.GestionnaireErreur;

/**
 * Cette classe contient l'ensemble des paramètres de configuration de NAT et
 * gère la sauvegarde et la récupération des différentes configurations
 * @author Bruno Mascret
 */
public class ConfigNat implements Serializable
{
    /** Filter Properties */
    private Properties fiConf;
    /** UI Properties */
    private Properties uiConf;
    /** UI configuration filename */
    private static final String uiConfFilename = "nat-gui.conf";

    /** serial version UID */
    private static final long serialVersionUID = 1L;

    /** constante pour représenter le mode garder toutes les lignes vides */
    public static final int AllEmptyLinesMode = -1;
    /** constante pour représenter le mode supprimer toutes les lignes vides */
    public static final int NoEmptyLinesMode = 0;
    /** constante pour représenter le mode de gestion paramétrée des lignes vides */
    public static final int ParametricEmptyLinesMode = 1;
    /** constante contenant l'adresse du dico de coupure par défaut */
	private static final String dicoCoupDefautName = "xsl/dicts/hyph_fr_nat.dic";

    /** Le fichier contenant la configuration */
    private String fichierConf;
	
    //************* configuration générale **************
    /** Numéro de version */
    private final String version = "2.0rc";
    /** Nom de version long */
    private final String versionLong = "2.0rc";
    /** Nom de la version svn correspondante */
	private final String svnVersion = "r793";

    //************* configuration des fichiers E/S ********
    /** adresse du dernier fichier source utilisé */
    private String fsource="";
    /** adresse du dernier fichier cible (sortie) utilisé */
    private String fcible="";
    /** encoding du fichier source */
    private String sourceEncoding="automatique";
    /** encoding du fichier de sortie */
    private String sortieEncoding="UTF-8";
    /** l'instance singleton de ConfigNat*/  
    private static ConfigNat cn=null;
    
    /**
     * Constructor
     * Creates user-specific configuration folders
     */
    private ConfigNat()
    {
		/* user configuration folder */
		try
	    {
	        String cpn = (new File(ConfigNat.getUserConfigFolder()).getCanonicalPath());			   
	        boolean success = (new File(ConfigNat.getUserConfigFolder())).mkdir();
	        if (success){System.out.println("Directory: " + cpn + " created");} 
	    }
        catch (IOException ioe){}

        try
	    {
	        String cpn = (new File(ConfigNat.getUserConfigFilterFolder()).getCanonicalPath());			   
	        boolean success = (new File(ConfigNat.getUserConfigFilterFolder())).mkdir();
	        if (success){System.out.println("Directory: " + cpn + " created");} 
	    }
        catch (IOException ioe){}
    }
 
	
    /**
     * @return the path of the system-wide configuration filters folder
     */
    public static String getSystemConfigFilterFolder()
    {
		String scf = "configurations/";
		return scf;
    }
    /**
     * @return the path of the user-specific configuration folder 
     */
    public static String getUserConfigFolder()
    {
		String ucf = System.getProperty("user.home")+"/.nat-braille/";
		return ucf;
    }
  
    /**
     * @return the path of the user-specific configuration filters folder 
     */
    public static String getUserConfigFilterFolder()
    {
		String ucf = getUserConfigFolder() + "filters/";
		return ucf;
    }
	
    /*
     * Méthodes d'accès
     */
	
    /* général */

    /** @return {@link #versionLong} */
    public String getVersionLong(){return versionLong;}
    /** @return {@link #version}*/
    public String getVersion(){return version;}
    /** @return {@link #svnVersion}*/
    public String getSvnVersion() {return svnVersion;}

    /* Fichiers E/S */
    /** @return {@link #fsource}*/
    public String getFsource(){return fsource;}
    /** @param fs valeur pour {@link #fsource}*/
    public void setFsource(String fs){fsource = fs;}
    /** @return {@link #fcible}*/
    public String getFcible(){return fcible;}
    /** @param fc valeur pour {@link #fcible}*/
    public void setFcible(String fc){fcible = fc;}
    /** @param f valeur pour {@link #fichierConf}*/
    public void setFichierConf(String f){fichierConf = f;}
    /** @return {@link #fichierConf}*/
    public String getFichierConf(){return fichierConf;}
    /** @return le nom court de la configuration courante, par exemple default */
    public String getShortFichierConf()
    {
	    String name = (new File(fichierConf).getName());
	    return name;
    }
    /* Filter Configuration */
    /** @return encodage du fichier source*/
    public String getSourceEncoding(){return fiConf.getProperty(OptNames.en_in,"UTF-8");}
    /** @param se valeur pour l'encodage du fichier source*/
    public void setSourceEncoding(String se){fiConf.setProperty(OptNames.en_in, se);}
    /** @return encodage du fichier de sortie*/
    public String getSortieEncoding(){return fiConf.getProperty(OptNames.en_out,"UTF-8");}
    /** @param se valeur pour l'encodage du fichier de sortie*/
    public void setSortieEncoding(String se){fiConf.setProperty(OptNames.en_out, se);}
    
    /** @param lg valeur pour le niveau de log*/
    public void setNiveauLog(int lg){fiConf.setProperty(OptNames.ge_log_verbosity, (new Integer(lg)).toString());}
    /**
     * Change le niveau de log et répercute la modification au GestionnaireErreur ge
     * @param lg le nouveau niveau de log
     * @param ge le gestionnaire à mettre à jour
     */
    public void setNiveauLog(int lg, GestionnaireErreur ge)
    {
    	fiConf.setProperty(OptNames.ge_log_verbosity, (new Integer(lg)).toString());
    	if(ge != null){ge.setNiveauLog(lg);}
    }
    /** @return le niveau de log*/
    public int getNiveauLog(){return ((new Integer(fiConf.getProperty(OptNames.ge_log_verbosity,"1"))).intValue());}
    /** @return adresse de la dtd */
    public String getDTD(){return (fiConf.getProperty(OptNames.fi_dtd_filename,"./mmlents/windob.dtd"));}
    /** @param dtd valeur pour adresse de la dtd*/
    public void setDTD(String dtd){fiConf.setProperty(OptNames.fi_dtd_filename,dtd);} 
    /** @return adresse de la feuille xsl principale de transcription*/
    public String getXSL(){return (fiConf.getProperty(OptNames.fi_filter_filename,"./xsl/xsl.xsl"));}
    /** @param xslfn valeur pour l'adresse de la feuille xsl principale de transcription*/
    public void setXSL(String xslfn){fiConf.setProperty(OptNames.fi_filter_filename,xslfn);}
    /** @return le nom de la table Braille utilisée*/
    public String getTableBraille(){return (fiConf.getProperty(OptNames.fi_braille_table,"brailleUTF8"));}
    /**
     * Met à jour le nom de la table braille et copie la nouvelle table dans le
     * fichier Brltab.ent
     * méthode d'accès
     * @param tableBraille le nom de la table braille à utiliser
     */
    public void setTableBraille(String tableBraille)
    {
		fiConf.setProperty(OptNames.fi_braille_table,tableBraille);
		FileChannel in = null; // canal d'entrée
		FileChannel out = null; // canal de sortie
		try 
		{
			// Init
			in = new FileInputStream("xsl/tablesBraille/"+tableBraille+".ent").getChannel();
			out = new FileOutputStream("xsl/tablesBraille/Brltab.ent").getChannel();
			
			// Copie depuis le in vers le out
			in.transferTo(0, in.size(), out);
			in.close();
			out.close();
		}
		catch (IOException e){e.printStackTrace();} // n'importe quelle exception
    }
    /** @return nom de la configuration*/
    public String getName(){return (fiConf.getProperty(OptNames.fi_name,"* base *"));}
    /** @param name valeur pour le nom de la configuration*/
    public void setName(String name){fiConf.setProperty(OptNames.fi_name,name);}
    /** @return description de la configuration*/
    public String getInfos(){return (fiConf.getProperty(OptNames.fi_infos,"* configuration de base  * "));}    
    /** @param infos valeur pour la description de la configuration*/
    public void setInfos(String infos){fiConf.setProperty(OptNames.fi_infos,infos);}
    /** @return vrai si la configuration est une configuration système*/
    public boolean getIsSysConfig(){return ((new Boolean(fiConf.getProperty(OptNames.fi_is_sys_config,"false"))).booleanValue()); }
    /** @param a vrai si la configuration est une configuration système*/
    public void setIsSysConfig(boolean a){fiConf.setProperty(OptNames.fi_is_sys_config,Boolean.toString(a));}
    
    /** @return vrai si abreger*/
    public boolean getAbreger(){return ((new Boolean(fiConf.getProperty(OptNames.fi_litt_abbreg,"false"))).booleanValue()); }
    /** @param a valeur pour abreger*/
    public void setAbreger(boolean a){fiConf.setProperty(OptNames.fi_litt_abbreg,Boolean.toString(a));}
    /** @return vrai si traiter maths */
    public boolean getTraiterMaths(){return ((new Boolean(fiConf.getProperty(OptNames.fi_math_transcribe,"true"))).booleanValue()); }
    /** @param m traiter maths*/
    public void setTraiterMaths(boolean m){fiConf.setProperty(OptNames.fi_math_transcribe,Boolean.toString(m));}
    /** @return vrai si traiter littéraire */
    public boolean getTraiterLiteraire(){return ((new Boolean(fiConf.getProperty(OptNames.fi_litt_transcribe,"true"))).booleanValue());}
    /** @param l traiter littéraire */
    public void setTraiterLiteraire(boolean l){fiConf.setProperty(OptNames.fi_litt_transcribe,Boolean.toString(l));}
    /** @return traiter musique */
    public boolean getTraiterMusique(){return ((new Boolean(fiConf.getProperty(OptNames.fi_music_transcribe,"true"))).booleanValue()); }
    /** @param m traiter musique*/
    public void setTraiterMusique(boolean m){fiConf.setProperty(OptNames.fi_music_transcribe,Boolean.toString(m));}
    
    /** @return utilisation de la notation spécifique trigo */
    public boolean getMathTrigoSpec(){return ((new Boolean(fiConf.getProperty(OptNames.fi_math_use_trigo_spec,"true"))).booleanValue()); }
    /** @param m utilisation de la notation spécifique trigo */
    public void setMathTrigoSpec(boolean m){fiConf.setProperty(OptNames.fi_math_use_trigo_spec,Boolean.toString(m));}
    /** @return true si préfixage systématique des maths */
    public boolean getMathPrefixAlways() {return ((new Boolean(fiConf.getProperty(OptNames.fi_math_force_prefix,"false"))).booleanValue());}
    /** @param mp true si préfixage systématique des maths */
    public void setMathPrefixAlways(boolean mp){fiConf.setProperty(OptNames.fi_math_force_prefix,Boolean.toString(mp));}
    
    /** @param lg longueur de la ligne */
    public void setLongueurLigne(int lg){fiConf.setProperty(OptNames.fi_line_lenght, (new Integer(lg)).toString());}
    /** @return longueur de la ligne */
    public int getLongueurLigne(){return ((new Integer(fiConf.getProperty(OptNames.fi_line_lenght,"40"))).intValue());}
    /** @param ln nombre de lignes par page*/
    public void setNbLigne(int ln){fiConf.setProperty(OptNames.fi_line_number, (new Integer(ln)).toString());}
    /** @return nombre de lignes par page*/
    public int getNbLigne(){return ((new Integer(fiConf.getProperty(OptNames.fi_line_number,"40"))).intValue());}
    /** @param m activer la mise en page*/
    public void setMep(boolean m) {fiConf.setProperty(OptNames.pf_do_layout,Boolean.toString(m));}
    /** @return activer la mise en page*/
    public boolean getMep(){return ((new Boolean(fiConf.getProperty(OptNames.pf_do_layout,"true"))).booleanValue()); }
    /** @param c coupure active*/
    public void setCoupure(boolean c){fiConf.setProperty(OptNames.fi_hyphenation,Boolean.toString(c));	}
    /** @return coupure active */
    public boolean getCoupure(){return ((new Boolean(fiConf.getProperty(OptNames.fi_hyphenation,"true"))).booleanValue()); }
    
    /**@param c coupure littéraire*/
    public void setCoupureLit(boolean c){fiConf.setProperty(OptNames.fi_hyphenation_lit,Boolean.toString(c));	}
    /**@return coupure littéraire*/
    public boolean getCoupureLit(){return ((new Boolean(fiConf.getProperty(OptNames.fi_hyphenation_lit,"false"))).booleanValue()); }
    /**@param m mode sagouin*/
    public void setModeCoupureSagouin(boolean m){fiConf.setProperty(OptNames.fi_hyphenation_dirty,Boolean.toString(m));}
    /**@return mode sagouin*/
    public boolean getModeCoupureSagouin(){return ((new Boolean(fiConf.getProperty(OptNames.fi_hyphenation_dirty,"false"))).booleanValue());}
    /** @return adresse dico de coupure */
    public String getDicoCoup(){return (fiConf.getProperty(OptNames.fi_hyphenation_rulefile_name,"xsl/dicts/hyph_fr_nat.dic"));} 
    /** @param dc adresse dico de coupure*/
    public void setDicoCoup(String dc){fiConf.setProperty(OptNames.fi_hyphenation_rulefile_name,dc);}
    //public String getDicoDesc(){return (fiConf.getProperty(OptNames.fi_hyphenation_rulefile_desc,"Dictionnaire Open-Office, v2.0, 18/03/08"));} 
    //public void setDicoDesc(String dd){fiConf.setProperty(OptNames.fi_hyphenation_rulefile_desc,dd);}
    /** @return feuille pour g1*/
    public String getXSL_g1(){return (fiConf.getProperty(OptNames.fi_litt_fr_int_filter_filename,"fr-g1.xsl"));}    
    /** @param filename feuilel pour g1*/
    public void setXSL_g1(String filename){fiConf.setProperty(OptNames.fi_litt_fr_int_filter_filename,filename);}
    /** @return feuille pour g2*/
    public String getXSL_g2(){return (fiConf.getProperty(OptNames.fi_litt_fr_abbreg_filter_filename,"fr-g2.xsl"));}    
    /** @param filename feuilel pour g2*/
    public void setXSL_g2(String filename){fiConf.setProperty(OptNames.fi_litt_fr_abbreg_filter_filename,filename);}
    /** @return feuille pour maths*/
    public String getXSL_maths(){return (fiConf.getProperty(OptNames.fi_math_filter_filename,"fr-maths.xsl"));}    
    /** @param filename feuilel pour maths*/
    public void setXSL_maths(String filename){fiConf.setProperty(OptNames.fi_math_filter_filename,filename);}
    /** @return feuille pour musique*/
    public String getXSL_musique(){return (fiConf.getProperty(OptNames.fi_music_filter_filename,"musique.xsl"));}    
    /** @param filename feuilel pour musique*/
    public void setXSL_musique(String filename){fiConf.setProperty(OptNames.fi_music_filter_filename,filename);}
    /** @return LitMajDouble*/
    public boolean getLitMajDouble(){return ((new Boolean(fiConf.getProperty(OptNames.tr_litt_double_upper,"true"))).booleanValue());}
    /** @param lmd LitMajDouble*/
    public void setLitMajDouble(boolean lmd){fiConf.setProperty(OptNames.tr_litt_double_upper,Boolean.toString(lmd));}
    /** @return LitMajPassage*/
    public boolean getLitMajPassage(){return ((new Boolean(fiConf.getProperty(OptNames.tr_litt_part_upper,"true"))).booleanValue());}
    /** @param lmp LitMajPassage*/
    public void setLitMajPassage(boolean lmp){fiConf.setProperty(OptNames.tr_litt_part_upper,Boolean.toString(lmp));}
    /** @return LitMajMelange*/
    public boolean getLitMajMelange(){return ((new Boolean(fiConf.getProperty(OptNames.tr_litt_mixed_upper,"true"))).booleanValue());}
    /** @param lmp LitMajMelange*/
    public void setLitMajMelange(boolean lmp){fiConf.setProperty(OptNames.tr_litt_mixed_upper,Boolean.toString(lmp));}
    /** @return LitEvidenceMot*/
    public boolean getLitEvidenceMot(){return ((new Boolean(fiConf.getProperty(OptNames.tr_litt_word_emph,"true"))).booleanValue());}
    /** @param lmp LitEvidenceMot*/
    public void setLitEvidenceMot(boolean lmp){fiConf.setProperty(OptNames.tr_litt_word_emph,Boolean.toString(lmp));}
    /** @return LitEvidencePassage*/
    public boolean getLitEvidencePassage(){return ((new Boolean(fiConf.getProperty(OptNames.tr_litt_part_emph,"true"))).booleanValue());}
    /** @param lmp LitEvidencePassage*/
    public void setLitEvidencePassage(boolean lmp){fiConf.setProperty(OptNames.tr_litt_part_emph,Boolean.toString(lmp));}
    /** @return LitEvidenceDansMot*/
    public boolean getLitEvidenceDansMot(){return ((new Boolean(fiConf.getProperty(OptNames.tr_litt_in_word_emph,"true"))).booleanValue());}
    /** @param lmp LitEvidenceDansMot*/
    public void setLitEvidenceDansMot(boolean lmp){fiConf.setProperty(OptNames.tr_litt_in_word_emph,Boolean.toString(lmp));}
	
    /** @return MepModelignes*/
    public int getMepModelignes(){return ((new Integer(fiConf.getProperty(OptNames.pf_empty_line_mode,"3"))).intValue());}
    /** @param mml MepModelignes*/
    public void setMepModelignes(int mml){fiConf.setProperty(OptNames.pf_empty_line_mode, (new Integer(mml)).toString());}
	/** @return MepMinLigne1*/
    public int getMepMinLigne1(){return ((new Integer(fiConf.getProperty(OptNames.pf_min_empty_line_1,"2"))).intValue());}
    /** @param mml1 MepMinLigne1*/
    public void setMepMinLigne1(int mml1){fiConf.setProperty(OptNames.pf_min_empty_line_1, (new Integer(mml1)).toString());}
	/** @return MepMinLigne2*/
    public int getMepMinLigne2(){return ((new Integer(fiConf.getProperty(OptNames.pf_min_empty_line_2,"3"))).intValue());}
    /** @param mml2 MepMinLigne2*/
    public void setMepMinLigne2(int mml2){fiConf.setProperty(OptNames.pf_min_empty_line_2, (new Integer(mml2)).toString());}
	/** @return MepMinLigne3*/
    public int getMepMinLigne3(){return ((new Integer(fiConf.getProperty(OptNames.pf_min_empty_line_3,"4"))).intValue());}
    /** @param mml3 MepMinLigne3*/
    public void setMepMinLigne3(int mml3){fiConf.setProperty(OptNames.pf_min_empty_line_3, (new Integer(mml3)).toString());}
	/** @return SautPageFin*/
    public boolean getSautPageFin() {return ((new Boolean(fiConf.getProperty(OptNames.pf_add_form_feed,"true"))).booleanValue());}
    /** @param spf SautPageFin*/
    public void setSautPageFin(boolean spf){fiConf.setProperty(OptNames.pf_add_form_feed,Boolean.toString(spf));}
    /** @param n Numerotation*/
    public void setNumerotation(String n) {fiConf.setProperty(OptNames.pf_numbering_style,n);}
	/** @return Numerotation*/
    public String getNumerotation() {return fiConf.getProperty(OptNames.pf_numbering_style,"'nn'");}
	
    /** @return TitresStricts*/
    public boolean getTitresStricts() {return ((new Boolean(fiConf.getProperty(OptNames.pf_strict_titles,"true"))).booleanValue());}
    /** @param ts TitresStricts*/
    public void setTitresStricts(boolean ts) {fiConf.setProperty(OptNames.pf_strict_titles,Boolean.toString(ts));}
	/** @return NiveauxTitres*/
    public String getNiveauxTitres() {return fiConf.getProperty(OptNames.pf_titles_levels,"1,2,3,4,5,5,5,5,5");}
    /** @param levels NiveauxTitres
     * @throws NumberFormatException problème de format du niveau de titre*/
    public void setNiveauxTitres(String levels) throws NumberFormatException
	{
		int i=0;
		String[] decoup = levels.split(",");
		try{for(i=0;i<decoup.length;i++){Integer.parseInt(decoup[i]);}}
		catch(NumberFormatException nfe){throw nfe;}
		fiConf.setProperty(OptNames.pf_titles_levels,levels);
	}
	
	/*public int getMepRetraitPar(){return ((new Integer(fiConf.getProperty(OptNames.pf_par_indent,"2"))).intValue());}
	public void setMepRetraitPar(int mrp){fiConf.setProperty(OptNames.pf_par_indent, (new Integer(mrp)).toString());}*/
	/** @return LineariseTable*/
    public boolean getLineariseTable() {return ((new Boolean(fiConf.getProperty(OptNames.pf_linearise_table,"false"))).booleanValue());}
    /** @param lt LineariseTable*/
    public void setLineariseTable(boolean lt) {fiConf.setProperty(OptNames.pf_linearise_table,Boolean.toString(lt));}
    /** @param mcl MinCellLin*/
    public void setMinCellLin(int mcl) {fiConf.setProperty(OptNames.pf_min_cell_linearise,Integer.toString(mcl));}
	/** @return MinCellLin*/
    public int getMinCellLin() {return ((new Integer(fiConf.getProperty(OptNames.pf_min_cell_linearise,"4"))).intValue());}
 /**
     * Splits a string using a separator which is regarded as a single caracter if doubled
     * @return a String[] of splited values
     * @param  s   : the string
     * @param  sep : the separator
     */
    static public String[] intelliSplit(String s,String sep)
    {
		ArrayList<String> z = new ArrayList<String>();
		String curString="";
		String[] ca = s.split(""); // each letter is an array element
		int i=0;
		while (i<(ca.length-1))
		{
		    String c = ca[i];
		    String n = ca[i+1];
		    if (c.equals(sep))
		    {
				if (n.equals(sep))
				{ // double separateur => caractere simple
				    curString += c; 
				    i++;
				}
				else 
				{            // veritable separateur => nouvelle string
				    z.add(curString);
				    curString = new String();
				}
		    }
		    else{ curString += c;}// caractere normal
		    i++;
		}
		// dernier element
		if (i<=ca.length){curString += ca[i];}
		z.add(curString);
		return ((String[]) z.toArray (new String[0]));
    }
    /** @return Rajout*/
    public String getRajout() {return fiConf.getProperty(OptNames.pf_strings_addons,"'','','','','','','','','','','',''");}
    /** @param r Rajout*/
    public void setRajout(String r) {fiConf.setProperty(OptNames.pf_strings_addons,r);}
	/** @return RajoutCompte*/
    public String getRajoutCompte() {return fiConf.getProperty(OptNames.pf_strings_addons_count,"false,false,false,false,false,false,false,false,false,false,false,false");}
    /** @param rc RajoutCompte*/
    public void setRajoutCompte(String rc) {fiConf.setProperty(OptNames.pf_strings_addons_count,rc);}

    /* Options graphiques mais dépendantes de la configuration 
     * Graphical options but depending of the current configuration
     * */
    /** @param pe PoliceEditeur*/
    public void setPoliceEditeur(String pe){fiConf.setProperty(OptNames.ui_editor_font,pe);}
    /** @return PoliceEditeur*/
    public String getPoliceEditeur(){return (fiConf.getProperty(OptNames.ui_editor_font,"Braille Antoine"));}
    /** @param pe2 PoliceEditeur2*/
    public void setPolice2Editeur(String pe2){fiConf.setProperty(OptNames.ui_editor_font2,pe2);}
    /** @return PoliceEditeur2*/
    public String getPolice2Editeur(){return (fiConf.getProperty(OptNames.ui_editor_font2,"Courrier"));}
    /** @param t TaillePolice*/
    public void setTaillePolice(int t){fiConf.setProperty(OptNames.ui_editor_font_size, (new Integer(t)).toString());}
    /** @return TaillePolice*/
    public int getTaillePolice(){return ((new Integer(fiConf.getProperty(OptNames.ui_editor_font_size,"24"))).intValue());}
    /** @param tp2 TaillePolice2*/
    public void setTaillePolice2(int tp2){fiConf.setProperty(OptNames.ui_editor_font2_size,(new Integer(tp2)).toString());}
    /** @return TaillePolice2*/
    public int getTaillePolice2(){return ((new Integer(fiConf.getProperty(OptNames.ui_editor_font2_size,"24"))).intValue());}
    
    /** @param saxp SaxonAsXsltProcessor*/
    public void setSaxonAsXsltProcessor(boolean saxp){fiConf.setProperty(OptNames.tr_use_saxon_processor,Boolean.toString(saxp));}
    /** @return SaxonAsXsltProcessor*/
    public boolean getSaxonAsXsltProcessor(){return new Boolean(fiConf.getProperty(OptNames.tr_use_saxon_processor,"true")).booleanValue();}
    /** @param nlf NbLogFiles*/
    public void setNbLogFiles(int nlf) {fiConf.setProperty(OptNames.ad_nb_log_files,Integer.toString(nlf));}
	/** @return NbLogFiles*/
    public int getNbLogFiles() {return ((new Integer(fiConf.getProperty(OptNames.ad_nb_log_files,"3"))).intValue());}
    /** @param lfs LogFileSize*/
    public void setLogFileSize(int lfs) {fiConf.setProperty(OptNames.ad_log_file_size,Integer.toString(lfs));}
	/** @return LogFileSize*/
    public int getLogFileSize() {return ((new Integer(fiConf.getProperty(OptNames.ad_log_file_size,"10"))).intValue());}
	
    /** GUI options **/
    /** @param o OuvreEditeurApresTranscription*/
    public void setOuvreEditeurApresTranscription(boolean o){uiConf.setProperty(OptNames.ui_editor_auto_open,Boolean.toString(o));}
    /** @return OuvreEditeurApresTranscription*/
    public boolean getOuvrirEditeur(){return ((new Boolean(uiConf.getProperty(OptNames.ui_editor_auto_open,"true"))).booleanValue());} 
    /** @return AfficheLigneSecondaire*/
    public boolean getAfficheLigneSecondaire(){return ((new Boolean(uiConf.getProperty(OptNames.ui_editor_zone2_display,"true"))).booleanValue());}
    /** @param als AfficheLigneSecondaire*/
    public void setAfficheLigneSecondaire(boolean als){uiConf.setProperty(OptNames.ui_editor_zone2_display,Boolean.toString(als));}
    /** @return Editeur*/
    public String getEditeur(){return fiConf.getProperty(OptNames.ui_editor_external,"");}
    /** @param e Editeur*/
    public void setEditeur(String e){fiConf.setProperty(OptNames.ui_editor_external,e);}
    /** @return UseNatEditor*/
    public boolean getUseNatEditor() {return new Boolean(fiConf.getProperty(OptNames.ui_editor_nat,"true")).booleanValue();}
    /** @param une UseNatEditor*/
    public void setUseNatEditor(boolean une) {fiConf.setProperty(OptNames.ui_editor_nat,""+une);}
	/** @return UseDefaultEditor*/
    public boolean getUseDefaultEditor() {return new Boolean(fiConf.getProperty(OptNames.ui_editor_default,"false")).booleanValue();}
    /** @param ude UseDefaultEditor*/
    public void setUseDefaultEditor(boolean ude) {fiConf.setProperty(OptNames.ui_editor_default,""+ude);}
    /*public String getConfDir(){return (uiConf.getProperty(OptNames.ui_editor_external,"configurations"));}    
    public void setConfDir(String dir){uiConf.setProperty(OptNames.ui_editor_external,dir);}*/
    

    /* remember last nat session  */
	/** @return LastSource*/
    public String getLastSource(){return (uiConf.getProperty(OptNames.last_source_filename,""));}    
    /** @param filename LastSource*/
    public void setLastSource(String filename){uiConf.setProperty(OptNames.last_source_filename,filename);}
    /** @return LastSourceEncoding*/
    public String getLastSourceEncoding(){return (uiConf.getProperty(OptNames.last_source_encoding,"automatique"));}    
    /** @param enc LastSourceEncoding*/
    public void setLastSourceEncoding(String enc){uiConf.setProperty(OptNames.last_source_encoding,enc);}
    /** @return LastDest*/
    public String getLastDest(){return (uiConf.getProperty(OptNames.last_dest_filename,""));}    
    /** @param filename LastDest*/
    public void setLastDest(String filename){uiConf.setProperty(OptNames.last_dest_filename,filename);}
    /** @return LastDestEncoding*/
    public String getLastDestEncoding(){return (uiConf.getProperty(OptNames.last_dest_encoding,"automatique"));}    
    /** @param enc LastDestEncoding*/
    public void setLastDestEncoding(String enc){uiConf.setProperty(OptNames.last_dest_encoding,enc);}
    /** @param conffn LastFilterConfigurationFilename*/
    public void setLastFilterConfigurationFilename(String conffn){uiConf.setProperty(OptNames.last_filter_configuration_filename,conffn);}
    /** @return LastFilterConfigurationFilename*/
    public String getLastFilterConfigurationFilename(){return (uiConf.getProperty(OptNames.last_filter_configuration_filename));}

    /* emboss */
    /** @param te table embossage*/
    public void setTableEmbossage(String te)
    {
	    fiConf.setProperty(OptNames.pr_emboss_table,te);
	    FileChannel in = null; // canal d'entrée
	    FileChannel out = null; // canal de sortie
	    try 
	    {
		    // Init
		    in = new FileInputStream("xsl/tablesEmbosseuse/"+te+".ent").getChannel();
		    out = new FileOutputStream("xsl/tablesEmbosseuse/Embtab.ent").getChannel();
			
		    // Copie depuis le in vers le out
		    in.transferTo(0, in.size(), out);
		    in.close();
		    out.close();
	    }
	    catch (IOException e){e.printStackTrace();} // n'importe quelle exception}
    }
    /** @return table Embossage*/
    public String getTableEmbossage(){return (fiConf.getProperty(OptNames.pr_emboss_table,"brailleUTF8"));}
    /** @param o OS*/
    public void setOs(String o){fiConf.setProperty(OptNames.pr_os,o);}
    /** @return OS*/
    public String getOs(){return (fiConf.getProperty(OptNames.pr_os,"Linux"));}
    /** @param c Commande*/
    public void setCommande(String c){fiConf.setProperty(OptNames.pr_emboss_command,c);}
    /** @return Commande */
    public String getCommande(){return (fiConf.getProperty(OptNames.pr_emboss_command,""));}
    /** @return UtiliserCommandeEmbossage*/
    public boolean getUtiliserCommandeEmbossage(){return ((new Boolean(fiConf.getProperty(OptNames.pr_use_emboss_command,"false"))).booleanValue());}
    /** @param uce UtiliserCommandeEmbossage*/
    public void setUtiliserCommandeEmbossage(boolean uce){fiConf.setProperty(OptNames.pr_use_emboss_command,Boolean.toString(uce));}
    /** @return UtiliserEmbosseuse*/
    public boolean getUtiliserEmbosseuse() {return ((new Boolean(fiConf.getProperty(OptNames.pr_emboss_auto,"false"))).booleanValue());}
    /** @param uce UtiliserEmbosseuse*/
    public void setUtiliserEmbosseuse(boolean uce){fiConf.setProperty(OptNames.pr_emboss_auto,Boolean.toString(uce));}
    /** @param c print service*/
    public void setPrintService(String c)
    {
    	if(c!=null){fiConf.setProperty(OptNames.pr_emboss_print_service,c);}
    	else{fiConf.setProperty(OptNames.pr_emboss_print_service,"");}
    }
    /** @return nom du print service*/
    public String getPrintservice(){return (fiConf.getProperty(OptNames.pr_emboss_print_service,""));}
    /** @return MemoriserFenetre*/
    public boolean getMemoriserFenetre(){return new Boolean(uiConf.getProperty(OptNames.ui_remember_windows_size,"true")).booleanValue();}
    /** @param rms MemoriserFenetre*/
    public void setMemoriserFenetre(boolean rms){uiConf.setProperty(OptNames.ui_remember_windows_size,Boolean.toString(rms));}
    /** @return CentrerFenetre*/
    public boolean getCentrerFenetre() {return new Boolean(uiConf.getProperty(OptNames.ui_center_windows,"true")).booleanValue();}
    /** @param cf CentrerFenetre*/
    public void setCentrerFenetre(boolean cf){uiConf.setProperty(OptNames.ui_center_windows,Boolean.toString(cf));}
    /** @return HeightEditeur*/
    public int getHeightEditeur(){return new Integer(uiConf.getProperty(OptNames.ui_y_editor,"0").toString());}
    /** @param he HeightEditeur*/
    public void setHeightEditeur(int he){uiConf.setProperty(OptNames.ui_y_editor,""+he);}
    /** @return WidthEditeur*/
    public int getWidthEditeur(){return new Integer(uiConf.getProperty(OptNames.ui_x_editor,"0").toString());}
    /** @param we WidthEditeur*/
    public void setWidthEditeur(int we){uiConf.setProperty(OptNames.ui_x_editor,""+we);}
    /** @return SonPendantTranscription*/
    public boolean getSonPendantTranscription(){return new Boolean(uiConf.getProperty(OptNames.ui_sound_during_work,"false")).booleanValue();}
    /** @param spt SonPendantTranscription*/
    public void setSonPendantTranscription(boolean spt){uiConf.setProperty(OptNames.ui_sound_during_work,""+spt);}
    /** @return SonFinTranscription*/
    public boolean getSonFinTranscription() {return new Boolean(uiConf.getProperty(OptNames.ui_sound_at_end,"false")).booleanValue();}
    /** @param sft SonFinTranscription*/
    public void setSonFinTranscription(boolean sft){uiConf.setProperty(OptNames.ui_sound_at_end,""+sft);}
 
	
    /**
     * Enregistre la configuration par serialisation dans le fichier {@link ConfigNat#fichierConf}
     */
    public void sauvegarder()
    {
		saveUiConf();
		saveFilterConf(fichierConf);
    }
    /**
     * Donne l'adresse du dictionnaire de coupure par défaut
     * @return l'adresse du dictionnaire de coupure par défaut
     */
    public static String getDicoCoupDefaut() {return dicoCoupDefautName;}
    /**
     * Crée ou remplace une configuration à partir du fichier sérialisé 
     * fconf passé en parametre ou trouvé dans le fichier 
     * de conf de l'interface graphique. 
     * @param fconf ; si null, pris dans le fichier de conf d'interface.
     * @return Un objet {@link ConfigNat} créé à partir de fconf
     * <p>IOException Erreur E/S lors de la lecture du fichier fconf<p>
     * <p>ClassNotFoundException Erreur lors du cast de la sortie de <code>ObjectInputStream.readObject()</code></p>
     */
    public static ConfigNat charger(String fconf)
    {	
		if (cn == null){cn = new ConfigNat();}

		if (fconf == null)
	    {
			/* last config filename is to be found in the last gui config file */
			cn.loadUiConf();
			fconf = cn.getLastFilterConfigurationFilename();
	    }
	
		cn.loadFilterConf(fconf);
		cn.setFichierConf(fconf);
		
		/* remplacement des tables brailles */
		cn.setTableBraille(cn.getTableBraille());
		cn.setTableEmbossage(cn.getTableEmbossage());
	
		return cn;
    }
    /** Renvoie la configuration en cours 
     * @return {@link #cn}*/
    public static ConfigNat getCurrentConfig(){return cn;}
    /** load and save ui and filter configuration 
     * @param configfile adresse du fichier de conf**/
    public void loadFilterConf(String configfile)
    {
	    fiConf = new Properties();
	    if (configfile != null) 
        {
		    try {fiConf.load(new InputStreamReader(new FileInputStream(configfile),"UTF-8"));}
		    catch (IOException ioe) 
	        {
			    ioe.printStackTrace(); 
			    System.err.println("1-Exception while reading filter configuration file");
	        }
        }	
    }
    /** save configuration in file named configFile 
     * @param configFile adresse du fichier de conf*/
    public void saveFilterConf (String configFile)
    {

	    try {fiConf.store(new FileOutputStream(configFile), null);}
	    catch (IOException ioe) 
	    {
		    ioe.printStackTrace(); 
		    System.err.println("Exception while writing filter configuration file");
	    }
	    catch(NullPointerException npe)
	    {
		    System.err.println("Sauvegarde de la configuration impossible: pas de configuration choisie");
	    }
    }
    /** load user interface configuration file **/
    public void loadUiConf () 
    {
	    uiConf = new Properties();
 	    boolean fail=false;

	    try {uiConf.load(new InputStreamReader(new FileInputStream(getUserConfigFolder()+uiConfFilename),"UTF-8"));}
	    catch (IOException ioe)
	    {
		    System.err.println("Exception while reading UI configuration file "+getUserConfigFolder()+uiConfFilename + "; using default values");
		    setLastSource("license.txt");
		    fail=true;
	    }

	    if (fail){
		try {uiConf.load(new InputStreamReader(new FileInputStream("nat-gui.conf-initial"),"UTF-8"));}
		catch (IOException ioe){
		    System.err.println("raté !");
		}
	    }
	    setFsource(getLastSource());
	    //setSourceEncoding(getLastSourceEncoding());
	    setFcible(getLastDest());
	    //setSortieEncoding(getLastDestEncoding());
    }
    /** save user interface configuration file **/
    public void saveUiConf ()
    {
	    setLastSource(fsource);
	    setLastSourceEncoding(sourceEncoding);
	    setLastDest(fcible);
	    setLastDestEncoding(sortieEncoding);
	    try{setLastFilterConfigurationFilename(fichierConf);}
        catch(NullPointerException npe){System.err.println("Sauvegarde du nom du fichier de configuration de l'interface graphique impossible: pas de configuration choisie");}

	    try{uiConf.store(new FileOutputStream(getUserConfigFolder()+uiConfFilename), null);}
        catch (IOException ioe) {System.err.println("Exception while writing UI configuration file");}	
    }
}
 