/*
 * NAT - An universal Translator
 * Copyright (C) 2005 Bruno Mascret
 * Contact: bmascret@free.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package nat;

import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;

import javax.swing.UIManager;

import nat.ConfigNat;
import nat.convertisseur.Convertisseur;
import nat.convertisseur.ConvertisseurChaine;
import nat.presentateur.PresentateurMEP;
import nat.transcodeur.Transcodeur;
import nat.transcodeur.TranscodeurNormal;
import nat.presentateur.Presentateur;

import joptsimple.OptionSet;
import joptsimple.OptionException;

import gestionnaires.AfficheurLog;
import gestionnaires.GestionnaireErreur;
import ui.AfficheurConsole;
import ui.FenetrePrinc;
/**
 * Classe principale de l'application
 * @author bruno
 *
 */
public class Nat 
{
    //constantes
	/** Représente un niveau de verbosité des logs muet */
	public final static int LOG_AUCUN = 0;
	/** Représente un niveau de verbosité des logs très faible */
    public final static int LOG_SILENCIEUX = 1;
    /** Représente un niveau de verbosité des logs normal */
    public final static int LOG_NORMAL = 2;
    /** Représente un niveau de verbosité des logs verbeux */
    public final static int LOG_VERBEUX = 3;
    /** Représente un niveau de verbosité des logs verbeux avec les informations de débuggage */
    public final static int LOG_DEBUG = 4;
    /** Vrai si utilisation de l'API saxon comme processeur xsl */
	public static boolean Saxon = false;
	/** String contenant la licence de NAT (GPL) */
    private static String licence;
    /** Une instance de gestionnaire d'erreur */
    private GestionnaireErreur gest;

	/** Liste d'instances de transcription représentant les transcription à réaliser */
	private ArrayList<Transcription> transcriptions = new ArrayList<Transcription>();
	//TODO raph a remplacer par ArrayList<Transcription> transcriptions = new ArrayList<Transcription>(); ???
	/**
	 * Constructeur
	 * @param g Une instance de GestionnaireErreur
	 */
    public Nat(GestionnaireErreur g) 
    {
		licence = getLicence("","");
		gest = g;
		
		//initialisation de certaines propriétés
		UIManager.put("FileChooser.lookInLabelMnemonic", new Integer(KeyEvent.VK_R));
		System.setProperty("javax.xml.transform.TransformerFactory",
		"net.sf.saxon.TransformerFactoryImpl");
		System.setProperty("javax.xml.parsers.DocumentBuilderFactory","org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");
    }

    /* méthodes d'accès */
    /**
     * renvoie le nom du fichier de configuration
     * @return le nom du fichier de configuration
     */
    public String getFichierConf(){return ConfigNat.getCurrentConfig().getFichierConf();}	
	/**
	 * Renvoie une chaine contenant le numéro long de la version de NAT
	 * @return une chaine contenant le numéro long de version
	 */
    public String getVersionLong(){return ConfigNat.getCurrentConfig().getVersionLong();};
    /**
	 * Renvoie une chaine contenant le nom de version de NAT
	 * @return une chaine contenant le nom de version
	 */
    public String getVersion(){return ConfigNat.getCurrentConfig().getVersion();};
    /**
     * Renvoie l'instance de GestionnaireErreur
     * @return l'instance de GestionnaireErreur
     * @see Nat#gest
     */
	public GestionnaireErreur getGestionnaireErreur() {return gest;}
	
	/**
	 * Renvoie la licence de nat préfixée par prefixe et terminée par suffixe
	 * @param prefixe préfixe à insérer avant la licence (/* ou <!-- par exemple)
	 * @param suffixe suffixe à insérer après la licence (* / ou --> par exemple)
	 * @return la licence de NAT
	 */
    public static String getLicence(String prefixe, String suffixe)
    {
	licence =  prefixe + " * NAT - An universal Translator\n" +
	    "* Copyright (C) 2005 Bruno Mascret\n" +
	    "* Contact: bmascret@free.fr\n" +
	    "* \n" +
	    "* This program is free software; you can redistribute it and/or\n" +
	    "* modify it under the terms of the GNU General Public License\n" +
	    "* as published by the Free Software Foundation; either version 2\n" +
	    "* of the License.\n" +
	    "* \n" +
	    "* This program is distributed in the hope that it will be useful,\n" +
	    "* but WITHOUT ANY WARRANTY; without even the implied warranty of\n" +
	    "* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n" +
	    "* GNU General Public License for more details.\n" +
	    "* \n" +
	    "* You should have received a copy of the GNU General Public License\n" +
	    "* along with this program; if not, write to the Free Software\n" +
	    "* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.\n" +
	    suffixe;	
	return licence;
    }
    /**
     * Fait appel à la fabrique Transcription pour obtenir une instance de transcription pour la source <code>source</code>
     * et la sortie <code>cible</code>
     * @param sources les adresses des fichiers source
     * @param cibles les adresses des fichier de sortie
     * @return <code>true</code> si la fabrication a réussi
     */
    public boolean fabriqueTranscriptions(ArrayList<String> sources, ArrayList<String> cibles)
    {
		boolean retour = true;
		//on vide la liste
		transcriptions.removeAll(transcriptions);
		for(int i=0;i<sources.size();i++)
		{
			String source = sources.get(i);
			String cible = cibles.get(i);
			Transcription t = Transcription.fabriqueTranscription(source,cible,gest);
			if(t!=null){transcriptions.add(t);}
			else{retour=false;}
		}
		return retour;
    }
    /**
     * Lance le processus complet de transcription des instances de <code>transcription</code>
     * @return true si le scénario s'est déroulé normallement
     * @see Nat#transcriptions
     */
    public boolean lanceScenario()
    {
    	gest.setException(null);
    	boolean retour = true;
    	for(Transcription t : transcriptions)
    	{
	    	try
	    	{
	    		retour = retour & t.transcrire();
	    	}
	    	catch(OutOfMemoryError oome)
	    	{
	    		gest.setException(new Exception("mémoire",oome));
	    		gest.gestionErreur();
	    	}
    	}
		return retour;
    }
    /**
     * Appel à la méthode touveEncodingSource de Transcription
     * @param source le fichier source
     * @param gest le gestionnaire d'erreur
     * @return une chaîne correspondant à l'encodage du fichier source
     * @see Transcription#trouveEncodingSource(String, GestionnaireErreur)
     */
    public String trouveEncodingSource(String source, GestionnaireErreur gest){return Transcription.trouveEncodingSource(source, gest);}
    /**
     * Charge certaines options de la ligne de commande dans le singleton de ConfigNat 
     * @param options OptionSet des options
     */
    public static void loadCliOptions(OptionSet options)
    {	
		ConfigNat cc = ConfigNat.getCurrentConfig();
	
		String nom = OptNames.ge_log_verbosity;
		if (options.has(nom)){cc.setNiveauLog(((Integer) options.valueOf(nom)).intValue());}
	
		nom = OptNames.fi_braille_table;
		if (options.has(nom)){cc.setTableBraille(((String) options.valueOf(nom)));}
		
		nom = OptNames.fi_math_transcribe;
		if (options.has(nom)){cc.setTraiterMaths(((Boolean) options.valueOf(nom)).booleanValue());}
	
		nom = OptNames.fi_math_use_trigo_spec;
		if (options.has(nom)){cc.setMathTrigoSpec(((Boolean) options.valueOf(nom)).booleanValue());}
		
		nom = OptNames.fi_litt_transcribe;
		if (options.has(nom)){cc.setTraiterLiteraire(((Boolean) options.valueOf(nom)).booleanValue());}
	
		nom = OptNames.fi_litt_abbreg;
		if (options.has(nom)){cc.setAbreger(((Boolean) options.valueOf(nom)).booleanValue());}
	
		nom = OptNames.fi_music_transcribe;
		if (options.has(nom)){cc.setTraiterMusique(((Boolean) options.valueOf(nom)).booleanValue());}
	
		nom = OptNames.fi_hyphenation;
		if (options.has(nom)){cc.setCoupure(((Boolean) options.valueOf(nom)).booleanValue());}
	
		nom = OptNames.fi_hyphenation_dirty;
		if  (options.has(nom)){cc.setModeCoupureSagouin(((Boolean) options.valueOf(nom)).booleanValue());}
	
		nom = OptNames.fi_line_lenght;
		if (options.has(nom)){cc.setLongueurLigne(((Integer) options.valueOf(nom)).intValue());}
		
		nom = OptNames.en_in;
		if (options.has(nom)){cc.setSourceEncoding((String)options.valueOf(nom));}
	
		nom = OptNames.en_out;
		if (options.has(nom)){cc.setSortieEncoding((String)options.valueOf(nom));}
    }
    /**
     * Méthode main
     * Analyse la chaine de paramètres, lance ou non l'interface graphique, la transcription, etc
     * @param argv les paramètres de la méthode main
     */
    public static void main (String argv [])
    {
    	//System.setProperty("file.encoding","UTF-8");
    	// Gestionnaire d'erreur de base (non utilisé par interface graphique)
    	ConfigNat.charger(null);  
    	GestionnaireErreur gestErreur = new GestionnaireErreur(null,ConfigNat.getCurrentConfig().getNiveauLog());
    	AfficheurConsole ac = new AfficheurConsole();
    	gestErreur.addAfficheur(ac);
    	gestErreur.addAfficheur(new AfficheurLog());
    	OptionParserNat parser = new OptionParserNat();
    	
    	Nat nat = new Nat(gestErreur);
    	try
	    {
    		OptionSet options = parser.parse( argv );
    		if (options.has("gui"))
		    {	 
				FenetrePrinc fenetre = new FenetrePrinc(nat);
				fenetre.pack();
				fenetre.setVisible(true);
		    }
    		else if (options.has("f") && options.has("t"))
		    {
    			if (options.has("q")) {gestErreur.removeAfficheur(ac);}
				if (options.has("c")) {ConfigNat.charger(options.valueOf( "c" ).toString());}
				loadCliOptions(options);
				
    			String[] cl_from = options.valueOf( "f" ).toString().split(":");
    			String[] cl_to = options.valueOf( "t" ).toString().split(":");
    			
    			/* exécution dans la console ou en ligne de commande?*/
    			if(cl_from[0].equals("-"))
    			{
    				gestErreur.removeAfficheur(ac);
    				gestErreur.afficheMessage("\nLecture de l'entrée standard... Ctrl+C pour quitter \n",Nat.LOG_NORMAL);
    				while(true)
    				{
    					boolean fin=false;
    					String chaine="";
    					while(!fin)
    					{
    						try 
    						{
								char c = (char)(System.in.read());
								if(c=='\n')
								{
									fin=true;
								}
								else
								{
									chaine=chaine+c;
								}
							} 
    						catch (IOException e) {e.printStackTrace();}
    					}
    					Convertisseur c = new ConvertisseurChaine(chaine,"tmp/tmp.xml","UTF-8");
    					Transcodeur t=new TranscodeurNormal("tmp/tmp.xml","tmp/tmp.txt","UTF-8",gestErreur);
    					Presentateur p=new PresentateurMEP(gestErreur, "UTF-8", "tmp/tmp.txt", "tmp/out.txt", "brailleUTF8");
    					
    					c.convertir(gestErreur);
    					t.transcrire(gestErreur);
    					p.presenter();
    					try {
    						BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream("tmp/out.txt")));
							System.out.println(br.readLine());
						} catch (FileNotFoundException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
    				}
    			}
    			else
				{	
    				gestErreur.afficheMessage("conversion de "+cl_from+" vers "+cl_to,Nat.LOG_NORMAL);
    				//fabrication des listes
    				ArrayList<String> sources = new ArrayList<String>();
    				ArrayList<String> cibles = new ArrayList<String>();
    				for (int i=0;i<cl_from.length;i++)
    				{
    					sources.add(cl_from[i]);
    					if(i>=cl_to.length){cibles.add(cl_from[i]+".braille");}
    					else{cibles.add(cl_to[i]);}
    				}
					if (nat.fabriqueTranscriptions(sources, cibles)){nat.lanceScenario();}
					else
					{
						gestErreur.afficheMessage("\n**ERREUR: certain fichiers n'existe pas et ne pourront être transcrits", Nat.LOG_SILENCIEUX);
						nat.lanceScenario();
					}
				}
		    }
    		else 
		    {
    			try{parser.printCliUsage();}
    			catch(IOException ioe){gestErreur.afficheMessage("\nErreur d'affichage pour les options", Nat.LOG_SILENCIEUX);}
		    }
	    }
    	catch (OptionException ex)
	    {
    		System.err.println( "====" );
    		gestErreur.afficheMessage("\nProblème dans la ligne de commande", Nat.LOG_VERBEUX);
    		try{parser.printCliUsage();}
			catch(IOException ioe){gestErreur.afficheMessage("\nErreur d'affichage pour les options", Nat.LOG_SILENCIEUX);}
	    }
    }

}
